@@ -18,13 +18,18 @@
 # along with python-mpd2.  If not, see <http://www.gnu.org/licenses/>.
 
 import logging
+import re
 import socket
 import sys
 import warnings
 
-VERSION = (3, 0, 3)
+from enum import Enum
+
+
+VERSION = (3, 1, 0)
 HELLO_PREFIX = "OK MPD "
 ERROR_PREFIX = "ACK "
+ERROR_PATTERN = re.compile(r"\[(?P<errno>\d+)@(?P<offset>\d+)\]\s+{(?P<command>\w+)}\s+(?P<msg>.*)")
 SUCCESS = "OK"
 NEXT = "list_OK"
 
@@ -46,6 +51,24 @@ logger = logging.getLogger(__name__)
 logger.addHandler(NullHandler())
 
 
+# MPD Protocol errors as found in CommandError exceptions
+# https://github.com/MusicPlayerDaemon/MPD/blob/master/src/protocol/Ack.hxx
+class FailureResponseCode(Enum):
+    NOT_LIST = 1
+    ARG = 2
+    PASSWORD = 3
+    PERMISSION = 4
+    UNKNOWN = 5
+
+    NO_EXIST = 50
+    PLAYLIST_MAX = 51
+    SYSTEM = 52
+    PLAYLIST_LOAD = 53
+    UPDATE_ALREADY = 54
+    PLAYER_SYNC = 55
+    EXIST = 56
+
+
 class MPDError(Exception):
     pass
 
@@ -59,7 +82,18 @@ class ProtocolError(MPDError):
 
 
 class CommandError(MPDError):
-    pass
+    def __init__(self, error):
+        self.errno = None
+        self.offset = None
+        self.command = None
+        self.msg = None
+
+        match = ERROR_PATTERN.match(error)
+        if match:
+            self.errno = FailureResponseCode(int(match.group("errno")))
+            self.offset = int(match.group("offset"))
+            self.command = match.group("command")
+            self.msg = match.group("msg")
 
 
 class CommandListError(MPDError):
@@ -205,12 +239,20 @@ class MPDClientBase(object):
         obj = {}
         for key, value in self._parse_pairs(lines):
             key = key.lower()
-            if lookup_delimiter and not delimiters:
-                delimiters = [key]
+            if lookup_delimiter and key not in delimiters:
+                delimiters = delimiters + [key]
             if obj:
                 if key in delimiters:
-                    yield obj
-                    obj = {}
+                    if lookup_delimiter:
+                        if key in obj:
+                            yield obj
+                            obj = obj.copy()
+                            while delimiters[-1] != key:
+                                obj.pop(delimiters[-1], None)
+                                delimiters.pop()
+                    else:
+                        yield obj
+                        obj = {}
                 elif key in obj:
                     if not isinstance(obj[key], list):
                         obj[key] = [obj[key], value]
@@ -288,6 +330,7 @@ class MPDClientBase(object):
     @mpd_commands(
         "add",
         "addtagid",
+        "binarylimit",
         "clear",
         "clearerror",
         "cleartagid",
@@ -345,7 +388,7 @@ class MPDClientBase(object):
         "swap",
         "swapid",
         "toggleoutput",
-        "umount",
+        "unmount",
         "unsubscribe",
         "volume",
     )
@@ -590,7 +633,7 @@ class MPDClient(MPDClientBase):
                     self.disconnect()
                     raise ConnectionError(
                         "Connection lost while reading binary data: "
-                        "expected %d bytes, got %d" % (chunk_size, len(data))
+                        "expected %d bytes, got %d" % (chunk_size, len(value))
                     )
 
                 if self._rbfile.read(1) != b"\n":
@@ -745,7 +788,9 @@ class MPDClient(MPDClientBase):
                 DeprecationWarning,
             )
             self.timeout = timeout
-        if host.startswith("/"):
+        if host.startswith("@"):
+            host = "\0" + host[1:]
+        if host.startswith(("/", "\0")):
             self._sock = self._connect_unix(host)
         else:
             if port is None:
