@@ -12,6 +12,7 @@ import types
 import warnings
 
 import unittest
+from unittest import mock
 
 try:
     from twisted.python.failure import Failure
@@ -26,11 +27,6 @@ except ImportError:
 
 import asyncio
 
-try:
-    import mock
-except ImportError:
-    print("Please install mock from PyPI to run tests!")
-    sys.exit(1)
 
 # show deprecation warnings
 warnings.simplefilter("default")
@@ -55,7 +51,7 @@ class TestMPDClient(unittest.TestCase):
             # Create a new socket.socket() mock with default attributes,
             # each time we are calling it back (otherwise, it keeps set
             # attributes across calls).
-            # That's probablyy what we want, since reconnecting is like
+            # That's probably what we want, since reconnecting is like
             # reinitializing the entire connection, and so, the mock.
             mock.MagicMock(name="socket.socket")
         )
@@ -266,6 +262,31 @@ class TestMPDClient(unittest.TestCase):
         self.assertIsNone(self.client.moveoutput("My ALSA Device"))
         self.assertMPDReceived('moveoutput "My ALSA Device"\n')
 
+    def test_list_group(self):
+        self.MPDWillReturn(
+            "AlbumArtist: Kraftwerk\n",
+            "Date: 1970\n",
+            "Album: Tone Float (Unofficial)\n",
+            "Date: 1974\n",
+            "Album: Autobahn\n",
+            "Album: Autobahn (2009 Der Katalog)\n",
+            "OK\n"
+        )
+        grouped_list = self.client.list(
+            "album", "albumartist", "Kraftwerk", "group", "date", "group", "albumartist"
+        )
+        self.assertMPDReceived(
+            'list "album" "albumartist" "Kraftwerk" "group" "date" "group" "albumartist"\n'
+        )
+        self.assertEqual(
+            [
+                {'albumartist': 'Kraftwerk', 'date': '1970', 'album': 'Tone Float (Unofficial)'},
+                {'albumartist': 'Kraftwerk', 'date': '1974', 'album': 'Autobahn'},
+                {'albumartist': 'Kraftwerk', 'date': '1974', 'album': 'Autobahn (2009 Der Katalog)'}
+            ],
+            grouped_list
+        )
+
     def test_client_to_client(self):
         # client to client is at this time in beta!
 
@@ -1266,12 +1287,19 @@ class TestAsyncioMPD(unittest.TestCase):
         self.mockserver.expect_exchange([], hello_lines)
 
         self.client = mpd.asyncio.MPDClient()
-        self._await(self.client.connect(TEST_MPD_HOST, TEST_MPD_PORT, loop=self.loop))
+        self._await(self.client.connect(TEST_MPD_HOST, TEST_MPD_PORT))
 
         asyncio.open_connection.assert_called_with(
-            TEST_MPD_HOST, TEST_MPD_PORT, loop=self.loop
+            TEST_MPD_HOST, TEST_MPD_PORT
         )
 
+    def __del__(self):
+        # Clean up after init_client. (This works for now; if it causes
+        # trouble, change all init_client to a `with init_client` or use
+        # fixtures).
+        if hasattr(self, "client"):
+            self.client.disconnect()
+
     def _await(self, future):
         return self.loop.run_until_complete(future)
 
@@ -1489,6 +1517,77 @@ class TestAsyncioMPD(unittest.TestCase):
         self.mockserver.expect_exchange([b"expecting odd things\n"], [b""])
         self.assertRaises(AssertionError, self._await, self.client.status())
 
+    async def _test_idle(self):
+        self.mockserver.expect_exchange(
+                [b'idle "database"\n',],
+                [b"ACK [4@0] don't let you yet but you shouldn't care\n"],
+                )
+        self.mockserver.expect_exchange(
+                # code could be smarter and set __in_idle = False and not set
+                # noidle, but it's not *wrong* either
+                [b'noidle\n', b'password "1234"\n',],
+                [b"OK\n"],
+                )
+        self.mockserver.expect_exchange(
+                [b'idle "playlist"\n'],
+                [b'idle: playlist\n', b"OK\n"],
+                )
+        self.mockserver.expect_exchange(
+                [b'noidle\n', b'currentsong\n'],
+                # it's a bit brief but it'll be accepted
+                [b"OK\n", ],
+                )
+        self.mockserver.expect_exchange(
+                [b'idle "playlist"\n'],
+                [b"ACK [4@0] I don't let you observe for no other reason\n"],
+                )
+        self.mockserver.expect_exchange(
+                [b'noidle\n', b'status\n'],
+                [b"ACK [4@0] whatever made the idle failed now fails too\n", ],
+                )
+
+        # clearly longer than IMMEDITE_COMMAND_TIMEOUT
+        await asyncio.sleep(0.5)
+
+        await self.client.password("1234")
+
+        async for changeset in self.client.idle(["playlist"]):
+            # the one regular event sent
+            self.assertEqual(changeset, ["playlist"])
+            break
+
+        await self.client.currentsong()
+
+        async for changeset in self.client.idle(["playlist"]):
+            # watching for regular events but the error also gets shown as an event
+            self.assertEqual(changeset, ["playlist"])
+            break
+
+        try:
+            await self.client.status()
+        except mpd.CommandError:
+            pass
+        else:
+            raise AssertionError("The status should have flushed out the error that made the idle fail in the first place")
+
+        self.client.disconnect()
+
+    def test_idle(self):
+        self.init_client()
+        self._await(self._test_idle())
+
+    async def _test_idle_timeout(self):
+        self.mockserver.expect_exchange([b'currentsong\n'], [b"OK\n"])
+        self.mockserver.expect_exchange([b'currentsong\n'], [b"OK\n"])
+        await self.client.currentsong()
+        # pausing for exactly this duration triggers a special case in mpd.asyncio.MPDClient.__run
+        await asyncio.sleep(self.client.IMMEDIATE_COMMAND_TIMEOUT)
+        await self.client.currentsong()
+        self.client.disconnect()
+
+    def test_idle_timeout(self):
+        self.init_client()
+        self._await(self._test_idle_timeout())
 
 if __name__ == "__main__":
     unittest.main()
